import fs, { promises } from "fs";
import { tmpdir } from "os";
import { dirname, join } from "path";
import { writeDir, writeDirSync } from "write-dir-safe";
import { removeFile } from "remove-file-safe";
import { fileExists, fileExistsSync } from "file-exists-safe";
import { addTerminatingNewline } from "terminating-newline";
function handleNewline(content, appendNewline) {
    if (appendNewline !== null && appendNewline !== void 0 ? appendNewline : true) {
        return addTerminatingNewline(content);
    } else {
        return content;
    }
}
let counter = 0;
async function openTemp() {
    const path = join(tmpdir(), `.${process.pid}.${counter}`);
    counter += 1;
    return promises.open(path, "wx").then((fd)=>{
        return {
            fd,
            path,
            cleanup: ()=>{
                fd.close().then(()=>{
                    removeFile(path);
                });
            }
        };
    }).catch((error)=>{
        if (error && error.code === "EEXIST") {
            return openTemp();
        } else {
            return undefined;
        }
    });
}
export async function writeFile(path, content = "", options = {}) {
    if (options.overwrite === false) {
        const exists = await fileExists(path);
        if (exists) {
            return true;
        }
    }
    return openTemp().then((temp)=>{
        if (temp) {
            return promises.writeFile(temp.fd, handleNewline(content, options.appendNewline)).then(async ()=>{
                const directory = dirname(path);
                var _options_recursive;
                if ((_options_recursive = options.recursive) !== null && _options_recursive !== void 0 ? _options_recursive : true) {
                    await writeDir(directory);
                }
                return promises.rename(temp.path, path).then(()=>{
                    return true;
                }).catch(()=>{
                    return false;
                });
            }).finally(()=>{
                temp.cleanup();
            });
        } else {
            return false;
        }
    });
}
export function writeFileSync(path, content = "", options = {}) {
    if (options.overwrite === false) {
        const exists = fileExistsSync(path);
        if (exists) {
            return true;
        }
    }
    const directory = dirname(path);
    var _options_recursive;
    if ((_options_recursive = options.recursive) !== null && _options_recursive !== void 0 ? _options_recursive : true) {
        writeDirSync(directory);
    }
    try {
        // eslint-disable-next-line no-sync
        fs.writeFileSync(path, handleNewline(content, options.appendNewline));
        return true;
    } catch (e) {
        return false;
    }
}
